/*
 * OCILIB - C Driver for Oracle (C Wrapper for Oracle OCI)
 *
 * Website: http://www.ocilib.net
 *
 * Copyright (c) 2007-2021 Vincent ROGIER <vince.rogier@ocilib.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include "ocilibcpp/core.hpp"

namespace ocilib
{
    namespace core
    {
        inline Locker::Locker() : _mutex(nullptr)
        {
            SetAccessMode(false);
        }

        inline Locker::~Locker() noexcept
        {
            SILENT_CATCH(SetAccessMode(false));
        }

        inline void Locker::SetAccessMode(bool threaded)
        {
            if (threaded && !_mutex)
            {
                _mutex = Mutex::Create();
            }
            else if (!threaded && _mutex)
            {
                Mutex::Destroy(_mutex);
                _mutex = nullptr;
            }
        }

        inline void Locker::Lock() const
        {
            if (_mutex)
            {
                Mutex::Acquire(_mutex);
            }
        }

        inline void Locker::Unlock() const
        {
            if (_mutex)
            {
                Mutex::Release(_mutex);
            }
        }

    }
}